"""
    1Password Connect

    REST API interface for 1Password Connect.  # noqa: E501

    The version of the OpenAPI document: 1.3.0
    Contact: support@1password.com
    Generated by: https://openapi-generator.tech
"""


from inspect import getfullargspec
import pprint

class GeneratorRecipe:
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'length': 'int',
        'character_sets': 'list[str]',
        'exclude_characters': 'str'
    }

    attribute_map = {
        'length': 'length',
        'character_sets': 'characterSets',
        'exclude_characters': 'excludeCharacters'
    }

    def __init__(self, length=32, character_sets=None, exclude_characters=None, local_vars_configuration=None):  # noqa: E501
        """GeneratorRecipe - a model defined in OpenAPI"""  # noqa: E501

        self._length = None
        self._character_sets = None
        self._exclude_characters = None
        self.discriminator = None

        if length is not None:
            self.length = length
        if character_sets is not None:
            self.character_sets = character_sets
        if exclude_characters is not None:
            self.exclude_characters = exclude_characters

    @property
    def length(self):
        """Gets the length of this GeneratorRecipe.  # noqa: E501

        Length of the generated value  # noqa: E501

        :return: The length of this GeneratorRecipe.  # noqa: E501
        :rtype: int
        """
        return self._length

    @length.setter
    def length(self, length):
        """Sets the length of this GeneratorRecipe.

        Length of the generated value  # noqa: E501

        :param length: The length of this GeneratorRecipe.  # noqa: E501
        :type length: int
        """
        if (length is not None and length > 64):  # noqa: E501
            raise ValueError("Invalid value for `length`, must be a value less than or equal to `64`")  # noqa: E501
        if (length is not None and length < 1):  # noqa: E501
            raise ValueError("Invalid value for `length`, must be a value greater than or equal to `1`")  # noqa: E501

        self._length = length

    @property
    def character_sets(self):
        """Gets the character_sets of this GeneratorRecipe.  # noqa: E501


        :return: The character_sets of this GeneratorRecipe.  # noqa: E501
        :rtype: list[str]
        """
        return self._character_sets

    @character_sets.setter
    def character_sets(self, character_sets):
        """Sets the character_sets of this GeneratorRecipe.


        :param character_sets: The character_sets of this GeneratorRecipe.  # noqa: E501
        :type character_sets: list[str]
        """
        allowed_values = ["LETTERS", "DIGITS", "SYMBOLS"]  # noqa: E501
        if (not set(character_sets).issubset(set(allowed_values))):  # noqa: E501
            raise ValueError(
                "Invalid values for `character_sets` [{}], must be a subset of [{}]"  # noqa: E501
                .format(", ".join(map(str, set(character_sets) - set(allowed_values))),  # noqa: E501
                        ", ".join(map(str, allowed_values)))
            )

        self._character_sets = character_sets

    @property
    def exclude_characters(self):
        """Gets the exclude_characters of this GeneratorRecipe.


        :return: The exclude_characters of this GeneratorRecipe.
        :rtype: str
        """
        return self._exclude_characters
    
    @exclude_characters.setter
    def exclude_characters(self, exclude_characters):
        """Sets the exclude_characters of this GeneratorRecipe.


        :param exclude_characters: The exclude_characters of this GeneratorRecipe.
        :type character_sets: str
        """
        duplicates = self.find_duplicates(exclude_characters)
        if duplicates:
            raise ValueError(
                "Invalid values for `exclude_characters` {0}, must not contain duplicate characters"
                .format(", ".join(map(str, duplicates)))
            )

        self._exclude_characters = exclude_characters

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr in self.openapi_types:
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GeneratorRecipe):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, GeneratorRecipe):
            return True

        return self.to_dict() != other.to_dict()

    def find_duplicates(self, s):
        char_count = {}  # Dictionary to store count of each character
        duplicates = set()  # Set to store duplicate characters

        for char in s:
            char_count[char] = char_count.get(char, 0) + 1  # Increment count of char

        for char, count in char_count.items():
            if count > 1:
                duplicates.add(char)  # If count is more than 1, it's a duplicate

        return duplicates


"""
    1Password Connect

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.2.1
    Generated by: https://openapi-generator.tech
"""


import pprint


class Field:
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'section': 'FieldSection',
        'type': 'str',
        'purpose': 'str',
        'label': 'str',
        'value': 'str',
        'generate': 'bool',
        'recipe': 'GeneratorRecipe',
        'entropy': 'float',
        'totp': 'str'
    }

    attribute_map = {
        'id': 'id',
        'section': 'section',
        'type': 'type',
        'purpose': 'purpose',
        'label': 'label',
        'value': 'value',
        'generate': 'generate',
        'recipe': 'recipe',
        'entropy': 'entropy',
        'totp': 'totp'
    }

    def __init__(self, id=None, section=None, type='STRING', purpose=None, label=None, value=None, generate=False, recipe=None, entropy=None, totp=None):  # noqa: E501
        self._id = None
        self._section = None
        self._type = None
        self._purpose = None
        self._label = None
        self._value = None
        self._generate = None
        self._recipe = None
        self._entropy = None
        self._totp = None
        self.discriminator = None

        self.id = id
        if section is not None:
            self.section = section
        if type is not None:
            self.type = type
        if purpose is not None:
            self.purpose = purpose
        if label is not None:
            self.label = label
        if value is not None:
            self.value = value
        if generate is not None:
            self.generate = generate
        if recipe is not None:
            self.recipe = recipe
        if entropy is not None:
            self.entropy = entropy
        if totp is not None:
            self.totp = totp

    @property
    def id(self):
        """Gets the id of this Field.  # noqa: E501


        :return: The id of this Field.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Field.


        :param id: The id of this Field.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def section(self):
        """Gets the section of this Field.  # noqa: E501


        :return: The section of this Field.  # noqa: E501
        :rtype: FieldSection
        """
        return self._section

    @section.setter
    def section(self, section):
        """Sets the section of this Field.


        :param section: The section of this Field.  # noqa: E501
        :type: FieldSection
        """

        self._section = section

    @property
    def type(self):
        """Gets the type of this Field.  # noqa: E501


        :return: The type of this Field.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Field.


        :param type: The type of this Field.  # noqa: E501
        :type: str
        """
        self._type = type

    @property
    def purpose(self):
        """Gets the purpose of this Field.  # noqa: E501

        Some item types, Login and Password, have fields used for autofill. This property indicates that purpose.  # noqa: E501

        :return: The purpose of this Field.  # noqa: E501
        :rtype: str
        """
        return self._purpose

    @purpose.setter
    def purpose(self, purpose):
        """Sets the purpose of this Field.

        Some item types, Login and Password, have fields used for autofill. This property indicates that purpose.  # noqa: E501

        :param purpose: The purpose of this Field.  # noqa: E501
        :type: str
        """
        allowed_values = ["", "USERNAME", "PASSWORD", "NOTES"]  # noqa: E501
        if purpose not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `purpose` ({}), must be one of {}"  # noqa: E501
                .format(purpose, allowed_values)
            )

        self._purpose = purpose

    @property
    def label(self):
        """Gets the label of this Field.  # noqa: E501


        :return: The label of this Field.  # noqa: E501
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """Sets the label of this Field.


        :param label: The label of this Field.  # noqa: E501
        :type: str
        """

        self._label = label

    @property
    def value(self):
        """Gets the value of this Field.  # noqa: E501


        :return: The value of this Field.  # noqa: E501
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this Field.


        :param value: The value of this Field.  # noqa: E501
        :type: str
        """

        self._value = value

    @property
    def generate(self):
        """Gets the generate of this Field.  # noqa: E501

        If value is not present then a new value should be generated for this field  # noqa: E501

        :return: The generate of this Field.  # noqa: E501
        :rtype: bool
        """
        return self._generate

    @generate.setter
    def generate(self, generate):
        """Sets the generate of this Field.

        If value is not present then a new value should be generated for this field  # noqa: E501

        :param generate: The generate of this Field.  # noqa: E501
        :type: bool
        """

        self._generate = generate

    @property
    def recipe(self):
        """Gets the recipe of this Field.  # noqa: E501


        :return: The recipe of this Field.  # noqa: E501
        :rtype: GeneratorRecipe
        """
        return self._recipe

    @recipe.setter
    def recipe(self, recipe):
        """Sets the recipe of this Field.


        :param recipe: The recipe of this Field.  # noqa: E501
        :type: GeneratorRecipe
        """

        self._recipe = recipe

    @property
    def entropy(self):
        """Gets the entropy of this Field.  # noqa: E501

        For fields with a purpose of `PASSWORD` this is the entropy of the value  # noqa: E501

        :return: The entropy of this Field.  # noqa: E501
        :rtype: float
        """
        return self._entropy

    @entropy.setter
    def entropy(self, entropy):
        """Sets the entropy of this Field.

        For fields with a purpose of `PASSWORD` this is the entropy of the value  # noqa: E501

        :param entropy: The entropy of this Field.  # noqa: E501
        :type: float
        """

        self._entropy = entropy
    
    @property
    def totp(self):
        """Gets the TOTP. # noqa: E501
        
        For fields of type 'OTP' this is the one-time password code # noqa: E501

        :return: The TOTP of the Field. # noqa: E501
        :rtype: str
        """
        return self._totp
    
    @totp.setter
    def totp(self, totp):
        """Sets the totp of this Field.

        For fields of type 'OTP' this is the one-time password code # noqa: E501

        :param totp: The TOTP of this Field.  # noqa: E501
        :type: str
        """

        self._totp = totp

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr in self.openapi_types.keys():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Field):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Field):
            return True

        return self.to_dict() != other.to_dict()



# import models into model package
from onepasswordconnectsdk.models.error import Error
from onepasswordconnectsdk.models.item import Item
from onepasswordconnectsdk.models.item_details import ItemDetails
from onepasswordconnectsdk.models.field import Field
from onepasswordconnectsdk.models.field_section import FieldSection
from onepasswordconnectsdk.models.file import File
from onepasswordconnectsdk.models.generator_recipe import GeneratorRecipe
from onepasswordconnectsdk.models.section import Section
from onepasswordconnectsdk.models.summary_item import SummaryItem
from onepasswordconnectsdk.models.item_urls import ItemUrls
from onepasswordconnectsdk.models.item_vault import ItemVault
from onepasswordconnectsdk.models.parsed_field import ParsedField
from onepasswordconnectsdk.models.parsed_item import ParsedItem
from onepasswordconnectsdk.models.vault import Vault

__all__ = [
    "Error",
    "Field",
    "FieldSection",
    "File",
    "GeneratorRecipe",
    "Item",
    "ItemDetails",
    "ItemUrls",
    "ItemVault",
    "ParsedField",
    "ParsedItem",
    "Section",
    "SummaryItem",
    "Vault",
]


class OnePasswordConnectSDKError(RuntimeError):
    pass


class EnvironmentTokenNotSetException(OnePasswordConnectSDKError, TypeError):
    pass


class EnvironmentHostNotSetException(OnePasswordConnectSDKError, TypeError):
    pass


class FailedToRetrieveItemException(OnePasswordConnectSDKError):
    pass


class FailedToRetrieveVaultException(OnePasswordConnectSDKError):
    pass


class FailedToDeserializeException(OnePasswordConnectSDKError, TypeError):
    pass



"""
    1Password Connect

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.2.1
    Generated by: https://openapi-generator.tech
"""


import pprint


class Item:
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'title': 'str',
        'vault': 'ItemVault',
        'category': 'str',
        'urls': 'list[ItemUrls]',
        'favorite': 'bool',
        'tags': 'list[str]',
        'version': 'int',
        'trashed': 'bool',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'last_edited_by': 'str',
        'sections': 'list[Section]',
        'fields': 'list[Field]'
    }

    attribute_map = {
        'id': 'id',
        'title': 'title',
        'vault': 'vault',
        'category': 'category',
        'urls': 'urls',
        'favorite': 'favorite',
        'tags': 'tags',
        'version': 'version',
        'trashed': 'trashed',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt',
        'last_edited_by': 'lastEditedBy',
        'sections': 'sections',
        'fields': 'fields'
    }

    def __init__(self, id=None, title=None, vault=None, category=None, urls=None, favorite=False, tags=None, version=None, trashed=False, created_at=None, updated_at=None, last_edited_by=None, sections=None, fields=None):  # noqa: E501
        self._id = None
        self._title = None
        self._vault = None
        self._category = None
        self._urls = None
        self._favorite = None
        self._tags = None
        self._version = None
        self._trashed = None
        self._created_at = None
        self._updated_at = None
        self._last_edited_by = None
        self._sections = None
        self._fields = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if title is not None:
            self.title = title
        self.vault = vault
        if category is not None:
            self.category = category
        if urls is not None:
            self.urls = urls
        if favorite is not None:
            self.favorite = favorite
        if tags is not None:
            self.tags = tags
        if version is not None:
            self.version = version
        if trashed is not None:
            self.trashed = trashed
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if last_edited_by is not None:
            self.last_edited_by = last_edited_by
        if sections is not None:
            self.sections = sections
        if fields is not None:
            self.fields = fields

    @property
    def id(self):
        """Gets the id of this Item.  # noqa: E501


        :return: The id of this Item.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Item.


        :param id: The id of this Item.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def title(self):
        """Gets the title of this Item.  # noqa: E501


        :return: The title of this Item.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this Item.


        :param title: The title of this Item.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def vault(self):
        """Gets the vault of this Item.  # noqa: E501


        :return: The vault of this Item.  # noqa: E501
        :rtype: ItemVault
        """
        return self._vault

    @vault.setter
    def vault(self, vault):
        """Sets the vault of this Item.


        :param vault: The vault of this Item.  # noqa: E501
        :type: ItemVault
        """

        self._vault = vault

    @property
    def category(self):
        """Gets the category of this Item.  # noqa: E501


        :return: The category of this Item.  # noqa: E501
        :rtype: str
        """
        return self._category

    @category.setter
    def category(self, category):
        """Sets the category of this Item.


        :param category: The category of this Item.  # noqa: E501
        :type: str
        """
        self._category = category

    @property
    def urls(self):
        """Gets the urls of this Item.  # noqa: E501


        :return: The urls of this Item.  # noqa: E501
        :rtype: list[ItemUrls]
        """
        return self._urls

    @urls.setter
    def urls(self, urls):
        """Sets the urls of this Item.


        :param urls: The urls of this Item.  # noqa: E501
        :type: list[ItemUrls]
        """

        self._urls = urls

    @property
    def favorite(self):
        """Gets the favorite of this Item.  # noqa: E501


        :return: The favorite of this Item.  # noqa: E501
        :rtype: bool
        """
        return self._favorite

    @favorite.setter
    def favorite(self, favorite):
        """Sets the favorite of this Item.


        :param favorite: The favorite of this Item.  # noqa: E501
        :type: bool
        """

        self._favorite = favorite

    @property
    def tags(self):
        """Gets the tags of this Item.  # noqa: E501


        :return: The tags of this Item.  # noqa: E501
        :rtype: list[str]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this Item.


        :param tags: The tags of this Item.  # noqa: E501
        :type: list[str]
        """

        self._tags = tags

    @property
    def version(self):
        """Gets the version of this Item.  # noqa: E501


        :return: The version of this Item.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this Item.


        :param version: The version of this Item.  # noqa: E501
        :type: int
        """

        self._version = version

    @property
    def trashed(self):
        """Gets the trashed of this Item.  # noqa: E501


        :return: The trashed of this Item.  # noqa: E501
        :rtype: bool
        """
        return self._trashed

    @trashed.setter
    def trashed(self, trashed):
        """Sets the trashed of this Item.


        :param trashed: The trashed of this Item.  # noqa: E501
        :type: bool
        """

        self._trashed = trashed

    @property
    def created_at(self):
        """Gets the created_at of this Item.  # noqa: E501


        :return: The created_at of this Item.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Item.


        :param created_at: The created_at of this Item.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this Item.  # noqa: E501


        :return: The updated_at of this Item.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this Item.


        :param updated_at: The updated_at of this Item.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def last_edited_by(self):
        """Gets the last_edited_by of this Item.  # noqa: E501


        :return: The last_edited_by of this Item.  # noqa: E501
        :rtype: str
        """
        return self._last_edited_by

    @last_edited_by.setter
    def last_edited_by(self, last_edited_by):
        """Sets the last_edited_by of this Item.


        :param last_edited_by: The last_edited_by of this Item.  # noqa: E501
        :type: str
        """

        self._last_edited_by = last_edited_by

    @property
    def sections(self):
        """Gets the sections of this Item.  # noqa: E501


        :return: The sections of this Item.  # noqa: E501
        :rtype: list[Section]
        """
        return self._sections

    @sections.setter
    def sections(self, sections):
        """Sets the sections of this Item.


        :param sections: The sections of this Item.  # noqa: E501
        :type: list[Section]
        """

        self._sections = sections

    @property
    def fields(self):
        """Gets the fields of this Item.  # noqa: E501


        :return: The fields of this Item.  # noqa: E501
        :rtype: list[Field]
        """
        return self._fields

    @fields.setter
    def fields(self, fields):
        """Sets the fields of this Item.


        :param fields: The fields of this Item.  # noqa: E501
        :type: list[Field]
        """

        self._fields = fields

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr in self.openapi_types.keys():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Item):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Item):
            return True

        return self.to_dict() != other.to_dict()



from dataclasses import dataclass
from typing import List
from onepasswordconnectsdk.models import ParsedField


@dataclass
class ParsedItem:
    vault_uuid: str
    item_title: str
    fields: List[ParsedField]



import os

from httpx._client import DEFAULT_TIMEOUT_CONFIG, Timeout

UUIDLength = 26
ENV_CLIENT_REQUEST_TIMEOUT = "OP_CONNECT_CLIENT_REQ_TIMEOUT"


def is_valid_uuid(uuid):
    if len(uuid) is not UUIDLength:
        return False
    for c in uuid:
        valid = (c >= 'a' and c <= 'z') or (c >= '0' and c <= '9')
        if valid is False:
            return False
    return True


def build_headers(token: str):
    """Builds the headers needed to make a request to the server

    Returns:
        dict: The 1Password Connect API request headers
    """
    return {"Authorization": f"Bearer {token}", "Content-Type": "application/json"}


class PathBuilder:
    def __init__(self, version: str = "/v1"):
        self.path: str = version

    def build(self) -> str:
        return self.path

    def vaults(self, uuid: str = None) -> 'PathBuilder':
        self._append_path("vaults")
        if uuid is not None:
            self._append_path(uuid)
        return self

    def items(self, uuid: str = None) -> 'PathBuilder':
        self._append_path("items")
        if uuid is not None:
            self._append_path(uuid)
        return self

    def files(self, uuid: str = None) -> 'PathBuilder':
        self._append_path("files")
        if uuid is not None:
            self._append_path(uuid)
        return self

    def content(self) -> 'PathBuilder':
        self._append_path("content")
        return self

    def query(self, key: str, value: str) -> 'PathBuilder':
        key_value_pair = f"{key}={value}"
        self._append_path(query=key_value_pair)
        return self

    def _append_path(self, path_chunk: str = None, query: str = None) -> 'PathBuilder':
        if path_chunk is not None:
            self.path += f"/{path_chunk}"
        if query is not None:
            self.path += f"?{query}"


def get_timeout() -> Timeout:
    """Get the timeout to be used in the HTTP Client"""
    raw_timeout = os.getenv(ENV_CLIENT_REQUEST_TIMEOUT, '0.0')
    if raw_timeout == 'None':
        return Timeout(None)  # disable all timeouts
    elif raw_timeout.isnumeric():
        timeout = float(raw_timeout)
        return timeout if timeout else DEFAULT_TIMEOUT_CONFIG
    else:
        return DEFAULT_TIMEOUT_CONFIG



from dateutil.parser import parse
import json
import re
import datetime
import onepasswordconnectsdk
from onepasswordconnectsdk.errors import FailedToDeserializeException


class Serializer:
    PRIMITIVE_TYPES = (float, bool, bytes, str, int)
    NATIVE_TYPES_MAPPING = {
        "int": int,
        "float": float,
        "str": str,
        "bool": bool,
        "date": datetime.date,
        "datetime": datetime.datetime,
        "object": object,
    }

    def deserialize(self, response, response_type):
        """Deserializes response into an object.

        :param response: RESTResponse object to be deserialized.
        :param response_type: class literal for
            deserialized object, or string of class name.

        :return: deserialized object.
        """
        # fetch data from response object
        try:
            data = json.loads(response)
        except ValueError:
            data = response

        return self.__deserialize(data, response_type)

    def sanitize_for_serialization(self, obj):
        """Builds a JSON POST object.

        If obj is None, return None.
        If obj is str, int, long, float, bool, return directly.
        If obj is datetime.datetime, datetime.date convert to string
        in iso8601 format.
        If obj is list, sanitize each element in the list.
        If obj is dict, return the dict.
        If obj is OpenAPI model, return the properties dict.

        :param obj: The data to serialize.
        :return: The serialized form of data.
        """
        if obj is None:
            return None
        elif isinstance(obj, self.PRIMITIVE_TYPES):
            return obj
        elif isinstance(obj, list):
            return [self.sanitize_for_serialization(sub_obj) for sub_obj in obj]  # noqa: E501
        elif isinstance(obj, tuple):
            return tuple(self.sanitize_for_serialization(sub_obj) for sub_obj in obj)  # noqa: E501
        elif isinstance(obj, (datetime.datetime, datetime.date)):
            return obj.isoformat()

        if isinstance(obj, dict):
            obj_dict = obj
        else:
            # Convert model obj to dict except
            # attributes `openapi_types`, `attribute_map`
            # and attributes which value is not None.
            # Convert attribute name to json key in
            # model definition for request.
            obj_dict = {
                obj.attribute_map[attr]: getattr(obj, attr)
                for attr in obj.openapi_types.keys()
                if getattr(obj, attr) is not None
            }

        return {
            key: self.sanitize_for_serialization(val)
            for key, val in obj_dict.items()
        }

    def __deserialize(self, data, klass):
        """Deserializes dict, list, str into an object.

        :param data: dict, list or str.
        :param klass: class literal, or string of class name.

        :return: object.
        """
        if data is None:
            return None

        if type(klass) == str:
            if klass.startswith("list["):
                sub_kls = re.match(r"list\[(.*)\]", klass).group(1)
                return [self.__deserialize(sub_data, sub_kls) for sub_data in data]  # noqa: E501

            if klass.startswith("dict("):
                sub_kls = re.match(r"dict\(([^,]*), (.*)\)", klass).group(2)
                return {
                    k: self.__deserialize(v, sub_kls) for k, v in data.items()  # noqa: E501
                }

            # convert str to class
            if klass in self.NATIVE_TYPES_MAPPING:
                klass = self.NATIVE_TYPES_MAPPING[klass]
            else:
                klass = getattr(onepasswordconnectsdk.models, klass)

        if klass in self.PRIMITIVE_TYPES:
            return self.__deserialize_primitive(data, klass)
        elif klass == object:
            return self.__deserialize_object(data)
        elif klass == datetime.date:
            return self.__deserialize_date(data)
        elif klass == datetime.datetime:
            return self.__deserialize_datetime(data)
        else:
            return self.__deserialize_model(data, klass)

    def __deserialize_primitive(self, data, klass):
        """Deserializes string to primitive type.

        :param data: str.
        :param klass: class literal.

        :return: int, long, float, str, bool.
        """
        try:
            return klass(data)
        except UnicodeEncodeError:
            return str(data)
        except TypeError:
            return data

    def __deserialize_object(self, value):
        """Return an original value.

        :return: object.
        """
        return value

    def __deserialize_date(self, string):
        """Deserializes string to date.

        :param string: str.
        :return: date.
        """
        try:
            return parse(string).date()
        except ImportError:
            return string
        except ValueError:
            raise FailedToDeserializeException(
                f'Failed to parse `{0}`\
                 as date object".format(string)'
            )

    def __deserialize_datetime(self, string):
        """Deserializes string to datetime.

        The string should be in iso8601 datetime format.

        :param string: str.
        :return: datetime.
        """
        try:
            return parse(string)
        except ImportError:
            return string
        except ValueError:
            raise FailedToDeserializeException(
                f'Failed to parse `{0}`\
                 as date object".format(string)'
            )

    def __deserialize_model(self, data, klass):
        """Deserializes list or dict to model.

        :param data: dict, list.
        :param klass: class literal.
        :return: model object.
        """
        has_discriminator = False
        if (
                hasattr(klass, "get_real_child_model")
                and klass.discriminator_value_class_map
        ):
            has_discriminator = True

        if not klass.openapi_types and has_discriminator is False:
            return data

        kwargs = {}
        if (
                data is not None
                and klass.openapi_types is not None
                and isinstance(data, (list, dict))
        ):
            for attr, attr_type in klass.openapi_types.items():
                if klass.attribute_map[attr] in data:
                    value = data[klass.attribute_map[attr]]
                    kwargs[attr] = self.__deserialize(value, attr_type)

        instance = klass(**kwargs)

        if has_discriminator:
            klass_name = instance.get_real_child_model(data)
            if klass_name:
                instance = self.__deserialize(data, klass_name)
        return instance



"""Python Client for connecting to 1Password Connect"""
import httpx
from httpx import HTTPError, USE_CLIENT_DEFAULT
import json
from typing import Dict, List, Union, Optional
import os

from onepasswordconnectsdk.async_client import AsyncClient
from onepasswordconnectsdk.config import ClientConfig
from onepasswordconnectsdk.serializer import Serializer
from onepasswordconnectsdk.utils import build_headers, is_valid_uuid, PathBuilder, get_timeout
from onepasswordconnectsdk.errors import (
    FailedToRetrieveItemException,
    FailedToRetrieveVaultException,
    EnvironmentHostNotSetException,
    EnvironmentTokenNotSetException,
)
from onepasswordconnectsdk.models import File, Item, ItemVault, SummaryItem, Vault
from onepasswordconnectsdk.models.constants import CONNECT_HOST_ENV_VARIABLE

ENV_SERVICE_ACCOUNT_JWT_VARIABLE = "OP_CONNECT_TOKEN"
ENV_IS_ASYNC_CLIENT = "OP_CONNECT_CLIENT_ASYNC"


class Client:
    """Python Client Class"""

    def __init__(self, url: str, token: str, config: Optional[ClientConfig] = None) -> None:
        """Initialize client
        
        Args:
            url (str): The url of the 1Password Connect API
            token (str): The 1Password Service Account token
            config (Optional[ClientConfig]): Optional configuration for httpx client
        """
        self.url = url
        self.token = token
        self.config = config
        self.session = self.create_session(url, token)
        self.serializer = Serializer()

    def create_session(self, url: str, token: str) -> httpx.Client:
        headers = self.build_headers(token)
        timeout = get_timeout()
        
        if self.config:
            client_args = self.config.get_client_args(url, headers, timeout)
            return httpx.Client(**client_args)
            
        return httpx.Client(base_url=url, headers=headers, timeout=timeout)

    def build_headers(self, token: str) -> Dict[str, str]:
        return build_headers(token)

    def __del__(self) -> None:
        self.session.close()

    def get_file(self, file_id: str, item_id: str, vault_id: str) -> File:
        url = PathBuilder().vaults(vault_id).items(item_id).files(file_id).build()
        response = self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve item. Received {response.status_code}\
                                     for {url} with message: {response.json().get('message')}"
            )
        return self.serializer.deserialize(response.content, "File")

    def get_files(self, item_id: str, vault_id: str) -> List[File]:
        url = PathBuilder().vaults(vault_id).items(item_id).files().build()
        response = self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve item. Received {response.status_code}\
                             for {url} with message: {response.json().get('message')}"
            )
        return self.serializer.deserialize(response.content, "list[File]")

    def get_file_content(self, file_id: str, item_id: str, vault_id: str, content_path: str = None) -> Union[bytes, str]:
        url = content_path
        if content_path is None:
            url = PathBuilder().vaults(vault_id).items(item_id).files(file_id).content().build()
        response = self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve items. Received {response.status_code} \
                     for {url} with message: {response.json().get('message')}"
            )
        return response.content

    def download_file(self, file_id: str, item_id: str, vault_id: str, path: str) -> None:
        file_object = self.get_file(file_id, item_id, vault_id)
        filename = file_object.name or "1password_item_file.txt"
        content = self.get_file_content(file_id, item_id, vault_id, file_object.content_path)
        global_path = os.path.join(path, filename)

        file = open(global_path, "wb")
        file.write(content)
        file.close()

    def get_item(self, item: str, vault: str) -> Item:
        """Get a specific item

        Args:
            item (str): the id or title of the item to be fetched
            vault (str): the id or name of the vault in which to get the item from

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            Item object: The found item
        """

        vault_id = vault
        if not is_valid_uuid(vault):
            vault_id = self.get_vault_by_title(vault).id

        if is_valid_uuid(item):
            return self.get_item_by_id(item, vault_id)
        else:
            return self.get_item_by_title(item, vault_id)

    def get_item_by_id(self, item_id: str, vault_id: str) -> Item:
        """Get a specific item by uuid

        Args:
            item_id (str): The id of the item to be fetched
            vault_id (str): The id of the vault in which to get the item from

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            Item object: The found item
        """
        url = PathBuilder().vaults(vault_id).items(item_id).build()
        response = self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve item. Received {response.status_code}\
                     for {url} with message: {response.json().get('message')}"
            )
        return self.serializer.deserialize(response.content, "Item")

    def get_item_by_title(self, title: str, vault_id: str) -> Item:
        """Get a specific item by title

        Args:
            title (str): The title of the item to be fetched
            vault_id (str): The id of the vault in which to get the item from

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            Item object: The found item
        """
        filter_query = f'title eq "{title}"'
        url = PathBuilder().vaults(vault_id).items().query("filter", filter_query).build()
        response = self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve items. Received {response.status_code} \
                     for {url} with message: {response.json().get('message')}"
            )

        if len(response.json()) != 1:
            raise FailedToRetrieveItemException(
                f"Found {len(response.json())} items in vault {vault_id} with \
                    title {title}"
            )

        item_summary = self.serializer.deserialize(response.content, "list[SummaryItem]")[0]
        return self.get_item_by_id(item_summary.id, vault_id)

    def get_items(self, vault_id: str, filter_query: str = None) -> List[SummaryItem]:
        """Returns a list of item summaries for the specified vault

        Args:
            vault_id (str): The id of the vault in which to get the items from
            filter_query (str): A optional query statement. `title eq "Example Item"`

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            List[SummaryItem]: A list of summarized items
        """
        if filter_query is None:
            url = PathBuilder().vaults(vault_id).items().build()
        else:
            url = PathBuilder().vaults(vault_id).items().query("filter", filter_query).build()

        response = self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve items. Received {response.status_code} \
                     for {url} with message: {response.json().get('message')}"
            )

        return self.serializer.deserialize(response.content, "list[SummaryItem]")

    def delete_item(self, item_id: str, vault_id: str) -> None:
        """Deletes a specified item from a specified vault

        Args:
            item_id (str): The id of the item in which to delete the item from
            vault_id (str): The id of the vault in which to delete the item

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API
        """
        url = PathBuilder().vaults(vault_id).items(item_id).build()
        response = self.build_request("DELETE", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to delete item. Received {response.status_code}\
                     for {url} with message: {response.json().get('message')}"
            )

    def create_item(self, vault_id: str, item: Item) -> Item:
        """Creates an item at the specified vault

        Args:
            vault_id (str): The id of the vault in which add the item to
            item (Item): The item to create

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            Item: The created item
        """

        url = PathBuilder().vaults(vault_id).items().build()
        response = self.build_request("POST", url, item)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to post item. Received {response.status_code}\
                    for {url} with message: {response.json().get('message')}"
            )
        return self.serializer.deserialize(response.content, "Item")

    def update_item(self, item_uuid: str, vault_id: str, item: Item) -> Item:
        """Update the specified item at the specified vault.

        Args:
            item_uuid (str): The id of the item in which to update
            vault_id (str): The id of the vault in which to update the item
            item (Item): The updated item

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            Item: The updated item
        """
        url = PathBuilder().vaults(vault_id).items(item_uuid).build()
        item.id = item_uuid
        item.vault = ItemVault(id=vault_id)

        response = self.build_request("PUT", url, item)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to post item. Received {response.status_code}\
                    for {url} with message: {response.json().get('message')}"
            )
        return self.serializer.deserialize(response.content, "Item")

    def get_vault(self, vault_id: str) -> Vault:
        """Returns the vault with the given vault_id

        Args:
            vault_id (str): The id of the vault in which to fetch

        Raises:
            FailedToRetrieveVaultException: Thrown when a HTTP error is
            returned from the 1Password Connect API

        Returns:
            Vault: The specified vault
        """
        url = PathBuilder().vaults(vault_id).build()
        response = self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveVaultException(
                f"Unable to retrieve vault. Received {response.status_code} \
                     for {url} with message {response.json().get('message')}"
            )

        return self.serializer.deserialize(response.content, "Vault")

    def get_vault_by_title(self, name: str) -> Vault:
        """Returns the vault with the given name

        Args:
            name (str): The name of the vault in which to fetch

        Raises:
            FailedToRetrieveVaultException: Thrown when a HTTP error is
            returned from the 1Password Connect API

        Returns:
            Vault: The specified vault
        """
        filter_query = f'name eq "{name}"'
        url = PathBuilder().vaults().query("filter", filter_query).build()
        response = self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveVaultException(
                f"Unable to retrieve vaults. Received {response.status_code} \
                     for {url} with message {response.json().get('message')}"
            )

        if len(response.json()) != 1:
            raise FailedToRetrieveItemException(
                f"Found {len(response.json())} vaults with \
                    name {name}"
            )

        return self.serializer.deserialize(response.content, "list[Vault]")[0]

    def get_vaults(self) -> List[Vault]:
        """Returns all vaults for service account set in client

        Raises:
            FailedToRetrieveVaultException: Thrown when a HTTP error is
            returned from the 1Password Connect API

        Returns:
            List[Vault]: All vaults for the service account in use
        """
        url = PathBuilder().vaults().build()
        response = self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveVaultException(
                f"Unable to retrieve vaults. Received {response.status_code} \
                     for {url} with message {response.json().get('message')}"
            )

        return self.serializer.deserialize(response.content, "list[Vault]")

    def build_request(self, method: str, path: str, body=None) -> httpx.Response:
        """Builds a http request
        Parameters:
        method (str): The rest method to be used
        path (str): The request path
        body (str): The request body

        Returns:
        Response object: The request response
        """

        if body:
            sanitized_body = self.serializer.sanitize_for_serialization(body)
            response = self.session.request(method, path, json=sanitized_body)
        else:
            response = self.session.request(method, path)
        return response

    def deserialize(self, response, response_type):
        return self.serializer.deserialize(response, response_type)

    def sanitize_for_serialization(self, obj):
        return self.serializer.sanitize_for_serialization(obj)


def new_client(url: str, token: str, is_async: bool = False, config: Optional[ClientConfig] = None) -> Union[AsyncClient, Client]:
    """Builds a new client for interacting with 1Password Connect
    
    Args:
        url (str): The url of the 1Password Connect API
        token (str): The 1Password Service Account token
        is_async (bool): Initialize async or sync client
        config (Optional[ClientConfig]): Optional configuration for httpx client
        
    Returns:
        Union[AsyncClient, Client]: The 1Password Connect client
    """
    if is_async:
        return AsyncClient(url, token, config)
    return Client(url, token, config)


def new_client_from_environment(url: str = None) -> Union[AsyncClient, Client]:
    """Builds a new client for interacting with 1Password Connect
    using the OP_TOKEN environment variable

    Parameters:
    url: The url of the 1Password Connect API
    token: The 1Password Service Account token

    Returns:
    Client: The 1Password Connect client
    """
    token = os.environ.get(ENV_SERVICE_ACCOUNT_JWT_VARIABLE)
    is_async = os.environ.get(ENV_IS_ASYNC_CLIENT) == "True"

    if url is None:
        url = os.environ.get(CONNECT_HOST_ENV_VARIABLE)
        if url is None:
            raise EnvironmentHostNotSetException(
                f"{CONNECT_HOST_ENV_VARIABLE} environment variable is not set"
            )

    if token is None:
        raise EnvironmentTokenNotSetException(
            "There is no token available in the "
            f"{ENV_SERVICE_ACCOUNT_JWT_VARIABLE} variable"
        )

    return new_client(url, token, is_async)



"""
    1Password Connect

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.2.1
    Generated by: https://openapi-generator.tech
"""


import pprint


class ItemDetails:
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'sections': 'list[Section]',
        'fields': 'list[Field]'
    }

    attribute_map = {
        'sections': 'sections',
        'fields': 'fields'
    }

    def __init__(self, sections=None, fields=None):  # noqa: E501
        self._sections = None
        self._fields = None
        self.discriminator = None

        if sections is not None:
            self.sections = sections
        if fields is not None:
            self.fields = fields

    @property
    def sections(self):
        """Gets the sections of this ItemDetails.  # noqa: E501


        :return: The sections of this ItemDetails.  # noqa: E501
        :rtype: list[Section]
        """
        return self._sections

    @sections.setter
    def sections(self, sections):
        """Sets the sections of this ItemDetails.


        :param sections: The sections of this ItemDetails.  # noqa: E501
        :type: list[Section]
        """

        self._sections = sections

    @property
    def fields(self):
        """Gets the fields of this ItemDetails.  # noqa: E501


        :return: The fields of this ItemDetails.  # noqa: E501
        :rtype: list[Field]
        """
        return self._fields

    @fields.setter
    def fields(self, fields):
        """Sets the fields of this ItemDetails.


        :param fields: The fields of this ItemDetails.  # noqa: E501
        :type: list[Field]
        """

        self._fields = fields

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr in self.openapi_types.keys():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ItemDetails):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ItemDetails):
            return True

        return self.to_dict() != other.to_dict()



"""
    1Password Connect

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.2.1
    Generated by: https://openapi-generator.tech
"""


import pprint


class Vault:
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'name': 'str',
        'description': 'str',
        'attribute_version': 'int',
        'content_version': 'int',
        'items': 'int',
        'type': 'str',
        'created_at': 'datetime',
        'updated_at': 'datetime'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'description': 'description',
        'attribute_version': 'attributeVersion',
        'content_version': 'contentVersion',
        'items': 'items',
        'type': 'type',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt'
    }

    def __init__(self, id=None, name=None, description=None, attribute_version=None, content_version=None, items=None, type=None, created_at=None, updated_at=None):  # noqa: E501
        """Vault - a model defined in OpenAPI"""  # noqa: E501
        self._id = None
        self._name = None
        self._description = None
        self._attribute_version = None
        self._content_version = None
        self._items = None
        self._type = None
        self._created_at = None
        self._updated_at = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if description is not None:
            self.description = description
        if attribute_version is not None:
            self.attribute_version = attribute_version
        if content_version is not None:
            self.content_version = content_version
        if items is not None:
            self.items = items
        if type is not None:
            self.type = type
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at

    @property
    def id(self):
        """Gets the id of this Vault.  # noqa: E501


        :return: The id of this Vault.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Vault.


        :param id: The id of this Vault.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this Vault.  # noqa: E501


        :return: The name of this Vault.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Vault.


        :param name: The name of this Vault.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def description(self):
        """Gets the description of this Vault.  # noqa: E501


        :return: The description of this Vault.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Vault.


        :param description: The description of this Vault.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def attribute_version(self):
        """Gets the attribute_version of this Vault.  # noqa: E501

        The vault version  # noqa: E501

        :return: The attribute_version of this Vault.  # noqa: E501
        :rtype: int
        """
        return self._attribute_version

    @attribute_version.setter
    def attribute_version(self, attribute_version):
        """Sets the attribute_version of this Vault.

        The vault version  # noqa: E501

        :param attribute_version: The attribute_version of this Vault.  # noqa: E501
        :type: int
        """

        self._attribute_version = attribute_version

    @property
    def content_version(self):
        """Gets the content_version of this Vault.  # noqa: E501

        The version of the vault contents  # noqa: E501

        :return: The content_version of this Vault.  # noqa: E501
        :rtype: int
        """
        return self._content_version

    @content_version.setter
    def content_version(self, content_version):
        """Sets the content_version of this Vault.

        The version of the vault contents  # noqa: E501

        :param content_version: The content_version of this Vault.  # noqa: E501
        :type: int
        """

        self._content_version = content_version

    @property
    def items(self):
        """Gets the items of this Vault.  # noqa: E501

        Number of active items in the vault  # noqa: E501

        :return: The items of this Vault.  # noqa: E501
        :rtype: int
        """
        return self._items

    @items.setter
    def items(self, items):
        """Sets the items of this Vault.

        Number of active items in the vault  # noqa: E501

        :param items: The items of this Vault.  # noqa: E501
        :type: int
        """

        self._items = items

    @property
    def type(self):
        """Gets the type of this Vault.  # noqa: E501


        :return: The type of this Vault.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Vault.


        :param type: The type of this Vault.  # noqa: E501
        :type: str
        """
        allowed_values = ["USER_CREATED", "PERSONAL", "EVERYONE", "TRANSFER"]  # noqa: E501
        if type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `type` ({}), must be one of {}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def created_at(self):
        """Gets the created_at of this Vault.  # noqa: E501


        :return: The created_at of this Vault.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Vault.


        :param created_at: The created_at of this Vault.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this Vault.  # noqa: E501


        :return: The updated_at of this Vault.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this Vault.


        :param updated_at: The updated_at of this Vault.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr in self.openapi_types.keys():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Vault):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Vault):
            return True

        return self.to_dict() != other.to_dict()



from onepasswordconnectsdk import client
from onepasswordconnectsdk import models
from onepasswordconnectsdk.config import load
from onepasswordconnectsdk.config import load_dict
from onepasswordconnectsdk.client import new_client
from onepasswordconnectsdk.client import new_client_from_environment

__all__ = [
    "client",
    "load",
    "load_dict",
    "models",
    "new_client",
    "new_client_from_environment"
]



import pprint


class File:
    openapi_types = {
        'id': 'str',
        'name': 'str',
        'size': 'int',
        'content_path': 'str',
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'size': 'size',
        'content_path': 'content_path',
    }

    def __init__(self, id=None, name=None, size=None, content_path=None):  # noqa: E501
        self._id = None
        self._name = None
        self._size = None
        self._content_path = None

        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if size is not None:
            self.size = size
        if content_path is not None:
            self.content_path = content_path

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, id):
        self._id = id

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @property
    def size(self):
        return self._size

    @size.setter
    def size(self, size):
        self._size = size

    @property
    def content_path(self):
        return self._content_path

    @content_path.setter
    def content_path(self, content_path):

        self._content_path = content_path

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr in self.openapi_types.keys():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, File):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, File):
            return True

        return self.to_dict() != other.to_dict()



import os
import shlex
from typing import List, Dict, Optional, TYPE_CHECKING
import httpx

if TYPE_CHECKING:
    from onepasswordconnectsdk.client import Client
from onepasswordconnectsdk.models import (
    Item,
    ParsedField,
    ParsedItem,
    Section,
)
from onepasswordconnectsdk.models.constants import (
    ITEM_TAG,
    FIELD_TAG,
    VAULT_TAG,
    VAULT_ID_ENV_VARIABLE,
)


class ClientConfig:
    """Configuration class for 1Password Connect client.
    Inherits from httpx.BaseClient to support all httpx client options.
    """
    def __init__(self, ca_file: Optional[str] = None, **kwargs):
        """Initialize client configuration

        Args:
            ca_file (Optional[str]): Path to CA certificate file for SSL verification
            **kwargs: Additional httpx client options
        """
        self.ca_file = ca_file
        self.httpx_options = kwargs

    def get_client_args(self, base_url: str, headers: Dict[str, str], timeout: float) -> Dict:
        """Get arguments for httpx client initialization

        Args:
            base_url (str): Base URL for the client
            headers (Dict[str, str]): Headers to include in requests
            timeout (float): Request timeout in seconds

        Returns:
            Dict: Arguments for httpx client initialization
        """
        args = {
            'base_url': base_url,
            'headers': headers,
            'timeout': timeout,
        }
        
        # Set verify from ca_file first
        if self.ca_file:
            args['verify'] = self.ca_file
            
        # Allow httpx_options (including verify) to override
        args.update(self.httpx_options)
            
        return args


def load_dict(client: "Client", config: dict):
    """Load: Takes a dictionary with keys specifiying the user
    desired naming scheme of the values to return. Each key's
    value is a dictionary that includes information on where
    to find the item field value in 1Password.

    Example dictionary
    {
        "foo": {
            "opitem": "Name of item in 1Password",  # noqa: RST301
            "opfield": "section_name.field_name"
        },  # noqa: RST203
        "bar": {
            "opitem": "Name of another item in 1Password",  # noqa: RST301
            "opfield": ".field_name",
            "opvault" "some_vault_id"
        }  # noqa: RST201
    }  # noqa: RST201

    opitem (required): describes the name of the item to access from 1Password

    offield (required): describes the name of the field to access within
    the specified item

    opvault: Only required if OP_VAULT is not set. Used to decribe the
    vault in which to fetch the item

    Args:
        client (Client): An instantied 1Password Connect client.

        config (dict): A dict with user specfied names for keys. Each key's
        value is a dictionary that includes information on where to find
        the item field value in 1Password.

    Returns:
        dict: A dictionary of user specified keys with values retrieved
        from 1Password
    """

    items: dict = {}
    config_values: Dict[str, str] = {}

    for field, tags in config.items():
        item_tag = tags.get(ITEM_TAG)
        field_tag = tags.get(FIELD_TAG)
        vault_tag = tags.get(VAULT_TAG)
        item_vault = _vault_uuid_for_field(field=field, vault_tag=vault_tag)
        key = f"{item_vault}/{item_tag}"
        item: ParsedItem = items.get(key)  # type: ignore

        if item:
            item.fields.append(ParsedField(field, field_tag))
        else:
            item = ParsedItem(
                vault_uuid=item_vault,
                item_title=item_tag,
                fields=[ParsedField(field, field_tag)],
            )
        items[key] = item
    for item_key, item in items.items():
        _set_values_for_item(client=client,
                             parsed_item=item,
                             config_dict=config_values)

    return config_values


def load(client: "Client", config: object):
    """Load: Takes a an object with class attributes annotated with tags
    describing where to find desired fields in 1Password. Manipulates given object
    and fills attributes in with 1Password item field values.

    Example class object
    class Foo():
        foo: 'opitem:"Name of Item" opfield:section_name.field_name' # noqa: E501, RST301
        bar: 'opitem:"Name of another item" opfield:.field_name opvault:vault_id' # noqa: E501

    opitem (required): describes the name of the item to access from 1Password

    opfield (required): describes the name of the field to access
    within the specified item

    opvault: Only required if OP_VAULT is not set. Used to decribe
    the vault in which to fetch the item

    Args:
        client (Client): An instantied 1Password Connect client.
        config (object): An object of a custom annoted class.
    """
    items: dict = {}
    annotations = config.__class__.__annotations__

    for field, tags in annotations.items():
        parsed_tags = _parse_tags(tags)
        item_tag = parsed_tags.get(ITEM_TAG)
        field_tag = parsed_tags.get(FIELD_TAG)
        vault_tag = parsed_tags.get(VAULT_TAG)
        item_vault = _vault_uuid_for_field(field=field, vault_tag=vault_tag)
        key = f"{item_vault}/{item_tag}"
        item: ParsedItem = items.get(key)  # type: ignore

        if item:
            item.fields.append(ParsedField(field, field_tag))
        else:
            item = ParsedItem(
                vault_uuid=item_vault,
                item_title=item_tag,
                fields=[ParsedField(field, field_tag)],
            )
        items[key] = item

    for item_key, item in items.items():
        _set_values_for_item(client=client,
                             parsed_item=item,
                             config_object=config)

    return config


def _parse_tags(tags: str):
    """Takes a string of whitespace seperated tags
    and assigns them to a dict. Expects each tag to
    be of the format 'tag_name:tag_value'

    Args:
        tags (str): The tags to parse

    Returns:
        dict: The parsed tags
    """
    return dict(item.split(":") for item in shlex.split(tags))  # type: ignore


def _vault_uuid_for_field(field: str, vault_tag: dict):
    env_vault_uuid = os.environ.get(VAULT_ID_ENV_VARIABLE)
    if vault_tag:
        return vault_tag
    elif env_vault_uuid:
        return env_vault_uuid
    raise NoVaultSetForFieldException(
        f"There \
        is no vault for {field} field"
    )


def _set_values_for_item(
    client: "Client",
    parsed_item: ParsedItem,
    config_dict={},
    config_object: object = None,
):
    # Fetching the full item
    item: Item = client.get_item(parsed_item.item_title, parsed_item.vault_uuid)

    sections = _convert_sections_to_dict(item.sections)

    for parsed_field in parsed_item.fields:
        if parsed_field.tag is None:
            raise NoFieldTagSetForFieldException(
                f"There is no {FIELD_TAG} specified \
                for {parsed_field.name}"
            )

        path_parts = parsed_field.tag.split(".")
        if len(path_parts) != 2:
            raise InvalidFieldPathException(
                f"Invalid field path format for \
                {parsed_field.name}"
            )

        value_found = False
        for field in item.fields:
            try:
                section_id = field.section.id
            except AttributeError:
                section_id = None
                
            if field.label == path_parts[1]:
                if (
                    section_id is None
                    or (section_id == sections.get(path_parts[0]))
                    or path_parts[0] in sections.values()
                ):
                    value_found = True

                    if config_object:
                        setattr(config_object, parsed_field.name, field.value)
                    else:
                        config_dict[parsed_field.name] = field.value
                    break
        if not value_found:
            raise UnknownSectionAndFieldTag(
                f"There is no section {path_parts[0]} \
                for field {path_parts[1]}"
            )


def _convert_sections_to_dict(sections: List[Section]):
    if not sections:
        return {}

    section_dict = {section.label: section.id for section in sections}
    return section_dict


class ConfigurationError(RuntimeError):
    pass


class NoVaultSetForFieldException(ConfigurationError):
    pass


class NoFieldTagSetForFieldException(ConfigurationError):
    pass


class InvalidFieldPathException(ConfigurationError):
    pass


class UnknownSectionAndFieldTag(ConfigurationError):
    pass



"""Python AsyncClient for connecting to 1Password Connect"""
import httpx
from httpx import HTTPError
from typing import Dict, List, Union, Optional
import os

from onepasswordconnectsdk.serializer import Serializer
from onepasswordconnectsdk.config import ClientConfig
from onepasswordconnectsdk.utils import build_headers, is_valid_uuid, PathBuilder, get_timeout
from onepasswordconnectsdk.errors import (
    FailedToRetrieveItemException,
    FailedToRetrieveVaultException,
)
from onepasswordconnectsdk.models import File, Item, ItemVault, SummaryItem, Vault


class AsyncClient:
    """Python Async Client Class"""

    def __init__(self, url: str, token: str, config: Optional[ClientConfig] = None) -> None:
        """Initialize async client
        
        Args:
            url (str): The url of the 1Password Connect API
            token (str): The 1Password Service Account token
            config (Optional[ClientConfig]): Optional configuration for httpx client
        """
        self.url = url
        self.token = token
        self.config = config
        self.session = self.create_session(url, token)
        self.serializer = Serializer()

    def create_session(self, url: str, token: str) -> httpx.AsyncClient:
        headers = self.build_headers(token)
        timeout = get_timeout()
        
        if self.config:
            client_args = self.config.get_client_args(url, headers, timeout)
            return httpx.AsyncClient(**client_args)
            
        return httpx.AsyncClient(base_url=url, headers=headers, timeout=timeout)

    def build_headers(self, token: str) -> Dict[str, str]:
        return build_headers(token)

    async def __aexit__(self):
        await self.session.aclose()

    async def get_file(self, file_id: str, item_id: str, vault_id: str) -> File:
        url = PathBuilder().vaults(vault_id).items(item_id).files(file_id).build()
        response = await self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve item. Received {response.status_code}\
                                     for {url} with message: {response.json().get('message')}"
            )
        return self.serializer.deserialize(response.content, "File")

    async def get_files(self, item_id: str, vault_id: str) -> List[File]:
        url = PathBuilder().vaults(vault_id).items(item_id).files().build()
        response = await self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve item. Received {response.status_code}\
                             for {url} with message: {response.json().get('message')}"
            )
        return self.serializer.deserialize(response.content, "list[File]")

    async def get_file_content(self, file_id: str, item_id: str, vault_id: str, content_path: str = None) -> Union[bytes, str]:
        url = content_path
        if content_path is None:
            url = PathBuilder().vaults(vault_id).items(item_id).files(file_id).content().build()
        response = await self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve items. Received {response.status_code} \
                     for {url} with message: {response.json().get('message')}"
            )
        return response.content

    async def download_file(self, file_id: str, item_id: str, vault_id: str, path: str) -> None:
        file_object = await self.get_file(file_id, item_id, vault_id)
        filename = file_object.name or "1password_item_file.txt"
        content = await self.get_file_content(file_id, item_id, vault_id, file_object.content_path)
        global_path = os.path.join(path, filename)

        file = open(global_path, "wb")
        file.write(content)
        file.close()

    async def get_item(self, item: str, vault: str) -> Item:
        """Get a specific item

        Args:
            item (str): the id or title of the item to be fetched
            vault (str): the id or name of the vault in which to get the item from

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            Item object: The found item
        """

        vault_id = vault
        if not is_valid_uuid(vault):
            vault = await self.get_vault_by_title(vault)
            vault_id = vault.id

        if is_valid_uuid(item):
            return await self.get_item_by_id(item, vault_id)
        else:
            return await self.get_item_by_title(item, vault_id)

    async def get_item_by_id(self, item_id: str, vault_id: str) -> Item:
        """Get a specific item by uuid

        Args:
            item_id (str): The id of the item to be fetched
            vault_id (str): The id of the vault in which to get the item from

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            Item object: The found item
        """
        url = PathBuilder().vaults(vault_id).items(item_id).build()
        response = await self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve item. Received {response.status_code}\
                     for {url} with message: {response.json().get('message')}"
            )
        return self.serializer.deserialize(response.content, "Item")

    async def get_item_by_title(self, title: str, vault_id: str) -> Item:
        """Get a specific item by title

        Args:
            title (str): The title of the item to be fetched
            vault_id (str): The id of the vault in which to get the item from

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            Item object: The found item
        """
        filter_query = f'title eq "{title}"'
        url = PathBuilder().vaults(vault_id).items().query("filter", filter_query).build()
        response = await self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve items. Received {response.status_code} \
                     for {url} with message: {response.json().get('message')}"
            )

        if len(response.json()) != 1:
            raise FailedToRetrieveItemException(
                f"Found {len(response.json())} items in vault {vault_id} with \
                    title {title}"
            )

        item_summary = self.serializer.deserialize(response.content, "list[SummaryItem]")[0]
        return await self.get_item_by_id(item_summary.id, vault_id)

    async def get_items(self, vault_id: str, filter_query: str = None) -> List[SummaryItem]:
        """Returns a list of item summaries for the specified vault

        Args:
            vault_id (str): The id of the vault in which to get the items from

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            List[SummaryItem]: A list of summarized items
        """
        if filter_query is None:
            url = PathBuilder().vaults(vault_id).items().build()
        else:
            url = PathBuilder().vaults(vault_id).items().query("filter", filter_query).build()
        response = await self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to retrieve items. Received {response.status_code} \
                     for {url} with message: {response.json().get('message')}"
            )

        return self.serializer.deserialize(response.content, "list[SummaryItem]")

    async def delete_item(self, item_id: str, vault_id: str) -> None:
        """Deletes a specified item from a specified vault

        Args:
            item_id (str): The id of the item in which to delete the item from
            vault_id (str): The id of the vault in which to delete the item
            from

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API
        """
        url = PathBuilder().vaults(vault_id).items(item_id).build()
        response = await self.build_request("DELETE", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to delete item. Received {response.status_code}\
                     for {url} with message: {response.json().get('message')}"
            )

    async def create_item(self, vault_id: str, item: Item) -> Item:
        """Creates an item at the specified vault

        Args:
            vault_id (str): The id of the vault in which add the item to
            item (Item): The item to create

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            Item: The created item
        """

        url = PathBuilder().vaults(vault_id).items().build()
        response = await self.build_request("POST", url, item)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to post item. Received {response.status_code}\
                    for {url} with message: {response.json().get('message')}"
            )
        return self.serializer.deserialize(response.content, "Item")

    async def update_item(self, item_uuid: str, vault_id: str, item: Item) -> Item:
        """Update the specified item at the specified vault.

        Args:
            item_uuid (str): The id of the item in which to update
            vault_id (str): The id of the vault in which to update the item
            item (Item): The updated item

        Raises:
            FailedToRetrieveItemException: Thrown when a HTTP error is returned
            from the 1Password Connect API

        Returns:
            Item: The updated item
        """
        url = PathBuilder().vaults(vault_id).items(item_uuid).build()
        item.id = item_uuid
        item.vault = ItemVault(id=vault_id)

        response = await self.build_request("PUT", url, item)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveItemException(
                f"Unable to post item. Received {response.status_code}\
                    for {url} with message: {response.json().get('message')}"
            )
        return self.serializer.deserialize(response.content, "Item")

    async def get_vault(self, vault_id: str) -> Vault:
        """Returns the vault with the given vault_id

        Args:
            vault_id (str): The id of the vault in which to fetch

        Raises:
            FailedToRetrieveVaultException: Thrown when a HTTP error is
            returned from the 1Password Connect API

        Returns:
            Vault: The specified vault
        """
        url = PathBuilder().vaults(vault_id).build()
        response = await self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveVaultException(
                f"Unable to retrieve vault. Received {response.status_code} \
                     for {url} with message {response.json().get('message')}"
            )

        return self.serializer.deserialize(response.content, "Vault")

    async def get_vault_by_title(self, name: str) -> Vault:
        """Returns the vault with the given name

        Args:
            name (str): The name of the vault in which to fetch

        Raises:
            FailedToRetrieveVaultException: Thrown when a HTTP error is
            returned from the 1Password Connect API

        Returns:
            Vault: The specified vault
        """
        filter_query = f'name eq "{name}"'
        url = PathBuilder().vaults().query("filter", filter_query).build()
        response = await self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveVaultException(
                f"Unable to retrieve vaults. Received {response.status_code} \
                     for {url} with message {response.json().get('message')}"
            )

        if len(response.json()) != 1:
            raise FailedToRetrieveItemException(
                f"Found {len(response.json())} vaults with \
                    name {name}"
            )

        return self.serializer.deserialize(response.content, "list[Vault]")[0]

    async def get_vaults(self) -> List[Vault]:
        """Returns all vaults for service account set in client

        Raises:
            FailedToRetrieveVaultException: Thrown when a HTTP error is
            returned from the 1Password Connect API

        Returns:
            List[Vault]: All vaults for the service account in use
        """
        url = PathBuilder().vaults().build()
        response = await self.build_request("GET", url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise FailedToRetrieveVaultException(
                f"Unable to retrieve vaults. Received {response.status_code} \
                     for {url} with message {response.json().get('message')}"
            )

        return self.serializer.deserialize(response.content, "list[Vault]")

    def build_request(self, method: str, path: str, body=None) -> httpx.Response:
        """Builds a http request
        Parameters:
        method (str): The rest method to be used
        path (str): The request path
        body (str): The request body

        Returns:
        Response object: The request response
        """

        if body:
            sanitized_body = self.serializer.sanitize_for_serialization(body)
            response = self.session.request(method, path, json=sanitized_body)
        else:
            response = self.session.request(method, path)
        return response

    def deserialize(self, response, response_type):
        return self.serializer.deserialize(response, response_type)

    def sanitize_for_serialization(self, obj):
        return self.serializer.sanitize_for_serialization(obj)
